#!/usr/bin/perl
# This might need to be changed. Some system admins use /usr/local/bin/perl

# checkfiles.pl v1.0 (April 14, 2004)
# This program is part of the sfick (Simple File Integrity Checker) project.
# by Justin Stauffer - 2004, March 16
#
# This program compares the MD5 checksum of the current file to the original
# checksum which is stored on the CD. If the checksums do not match, the user
# will be prompted to replace the current file with a backup copy from the CD.
#
# This is the general process:
#
# 1)  Open the configuration file.
# 2)  Read the configuration file line by line parsing the string into
#       individual fields.
# 3)  Open the file on the hard drive, compute the digest of the contents, and
#       finally compare that digest to the original digest stored on the CD.
# 4)  If the digest is different, prompt the user for replacement.
# 5)  If the user chooses to replace the file, do so.
# 6)  Return to step 2 until EOF.


use Digest::MD5 qw(md5_hex);
use File::Copy;

# constants
$CONFIG_FILE = "config.cfg";
$DIGEST_EXT  = ".digest";
    

if(!open(CONFIG, $CONFIG_FILE)) {
    die "Error: Failed to open configuration file: $CONFIG_FILE\n";
}

print ("\nVerifying file integrity using configuration: \'$CONFIG_FILE\' ...\n\n");

$i = 1;

while(defined($line = <CONFIG>)) {
    if($line =~ /\n/) {   # if the line ends in a '\n', we remove it.
	chop($line);
    }

    # splitting the line into segments divided by ':'
    # field[0] is the file name
    # field[1] is the location of the file on the user's hard drive
    # field[2] is the location of the file's directory on the CD.  
    #
    # The file's directory stored on the CD will contain the following:
    # 1) a backup copy of the file for restoration purposes
    # 2) a digest file named: <filename>.digest

    @field = split(/:/, $line);

    print ("-" x 70 . " < $i >\n");

    print ("Verifying file: $field[0] \@ $field[1]\n");
   
    # Open the file on the hard drive.
    if(!open(FILE, $field[1])) {
	print ("Error: Could not open current file \'$field[1]\'.\nSkipping this file.\n");
    }
    else {
	# Read the entire file into a string.
	$file = undef;
	while(defined($tmp = <FILE>)) {
	    $file = $file . $tmp;
	}
	
	# Compute the hash digest of the file contents.
	$digest = md5_hex($file);

	# Open the corresponding hash digest stored on the CD.
	$filename =  "$field[2]/" . "$field[0]" . "$DIGEST_EXT";
    
	if(!open(DIGEST, $filename)) {
	    print ("Error: Could not open digest file \'$filename\'.\nSkipping this file.\n");
	}
	else {
	    # The digest should be one line but we'll read in all lines anyway.
	    $cd_digest = undef;
	    while(defined($tmp = <DIGEST>)) {
		# if the line ends in a '\n', we remove it.
		if($tmp =~ /\n/) {
		    chop($tmp);
		}

		$cd_digest = $cd_digest . $tmp;
	    }

	    print ("current digest: $digest\n");
	    print ("archive digest: $cd_digest\n");

	    # Do the digests match?
	    if($cd_digest ne $digest) {
		print ("\nDIGESTS DO NOT MATCH! The file has been altered.\n");
		print ("Do you want to replace the modified file? (yes/no): ");
		$replace = false;
		$backup = false;

		if(<STDIN> =~ /[Yy]+/) {
		    $replace = true;
		    $archived_file = $field[2] . "/" . $field[0];
		    
		    # The user may want to save a backup of the modifed file.
		    print "Do you want to backup the modified file? (yes/no): ";
		    if(<STDIN> =~ /[Yy]+/) {
			# The user wants to backup the file.
			$backup = true;

			# We want to backup the file to be replaced.
			# We append a random string to the new file name to
			# avoid symlink vulnerabilities.
			$backup_file = $field[1] . ".bak" . int(rand(1000000));

			if(!move($field[1], $backup_file)) {
			    # Backup failed. 
			    print ("-- Error: Failed to backup file.\n\n");
			    print ("Do you still want to replace the modified file? (yes/no): ");
			    if(<STDIN> =~ /[Yy]+/) {
				$replace = true;
			    }
			    else {
				$replace = false;
			    }
			}
			else {
			    print ("\nBacked up file: \"$field[1]\" to   \"$backup_file\"\n");
			}
		    }

		    # Make sure the user still wants to replace the file.

		    if($replace eq true) {
			# We can now proceed to copying the CD archived file
			# into the old file's location.

			if(copy($archived_file, $field[1])) {
			    print ("Replaced file:  \"$field[1]\" with \"$archived_file\".\n");
			}
			else {
			    print ("-- Error: Failed to restore archived file - copy failed.\n\n");
			    print ("Press [Enter] to continue...");
			    <STDIN>;
			}
		    }
		    else {
			print ("-- Restoration cancelled.\n");
		    }
		}
	    }
	    else {
		print ("Digests match.\n");
	    }

	} # end else
    } # end else

    # return to step 2.
    $i++;

} # end while


print ("-" x 70 . " done\n");
print ("\nVerification completed.\n\n");
